#ifndef _Advanced_genericity_H
#define _Advanced_genericity_H

#include <limits>

class Male;

class Female {
public:
	const std::string identity;
	// Refuse (implicit) 'Female = "SohieDarnal";':
	inline explicit Female(const std::string& identity) : identity(identity) {};
	// Refuse (implicit) 'f = m;' provided that assignment is supported as well:
	explicit operator Male() const;
	// Template member function (from C++11):
	template<typename Specie> explicit operator Specie() const { // Conversion to 'Specie', refuse "implicit"
		return Specie();
	};
};

class Male {
public:
	const std::string identity;
	// Refuse (implicit) 'Male = "FranckBarbier";':
	inline explicit Male(const std::string& identity) : identity(identity) {};
	// Accept (implicit) 'm = f;' provided that assignment is supported as well:
	operator Female() const;
	// g++ *REQUIRES* 'explicit' (compilation error) while Visual Studio doesn't:
	template<typename Specie> /*explicit*/ operator Specie() const { // Conversion to 'Specie', accept "implicit"
		return Specie();
	};
};

class Minotaur {
public:
	const std::string identity;
	inline Minotaur(const std::string& identity) : identity(identity) {};
};
// Explicit specialization for 'Minotaur':

template<> Male::operator Minotaur() const { // Because 'Specie()' cannot function...
	return Minotaur(identity);
}
// Default no-argument constructor to match to 'Specie()':
class Medusa { /* Snake woman */ };
class Sphinx { /* Lion woman */ };
// Explicit instantiation for 'Sphinx':
template Female::operator Sphinx() const;

template<typename T> void my_function(T& t = std::declval<T>()) {
	T other = t;
};
// Explicit specialization ('T' = 'int') -> *no default value for argument is permitted*:
template<> void my_function(int& t) {
	int* other = &t;
};
// Instantiation (declaration only since body presence means specialization!):
template void my_function(char& t);

/** Variable template */

// 'constexpr' is for access at compilation time:
template<typename Number> /* constexpr */ Number Mid = std::numeric_limits<Number>::max() - std::numeric_limits<Number>::min() / static_cast<Number> (2);
template<typename My_anonymous> class My_class {
public:
	// Declaration:
	template<class Number> static const Number Three_key_positive_values;
	static const My_anonymous My_class_property; // Non-template
};
// Definition:
template<typename My_anonymous> template<typename Number> const Number My_class<My_anonymous>::Three_key_positive_values = { std::numeric_limits<Number>::min(), Mid<Number>, std::numeric_limits<Number>::max() };
template<typename My_anonymous> const My_anonymous My_class<My_anonymous>::My_class_property = My_anonymous();
#endif /* _Advanced_genericity_H */

